from PySimultan2 import Content

contents = {}


contents['param_a'] = Content(text_or_key='param_a',
                              property_name='param_a',
                              type=None,
                              unit=None,
                              documentation='param_a')

contents['param_b'] = Content(text_or_key='param_b',
                              property_name='param_b',
                              type=None,
                              unit=None,
                              documentation='param_b')

contents['param_c'] = Content(text_or_key='param_c',
                              property_name='param_c',
                              type=None,
                              unit=None,
                              documentation='param_c')

contents['param_d'] = Content(text_or_key='param_d',
                              property_name='param_d',
                              type=None,
                              unit=None,
                              documentation='param_d')

contents['thermal_conductivity'] = Content(text_or_key='thermal_conductivity',
                                           property_name='thermal_conductivity',
                                           type=None,
                                           unit=None,
                                           documentation='thermal_conductivity')

contents['thermal_resistance'] = Content(text_or_key='thermal_resistance',
                                         property_name='thermal_resistance',
                                         type=None,
                                         unit=None,
                                         documentation='thermal_resistance')
